/*
 * Copyright 2006 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */
package com.sun.xml.internal.xsom.impl.util;

import java.util.Iterator;

/**
 * {@link Iterator} that works as a filter to another {@link Iterator}.
 * 
 * @author
 *     Kohsuke Kawaguchi (kohsuke.kawaguchi@sun.com)
 */
public abstract class FilterIterator implements Iterator {
    
    private final Iterator core;
    private Object next;
    
    protected FilterIterator( Iterator core ) {
        this.core = core;
    }
    
    /**
     * Implemented by the derived class to filter objects.
     * 
     * @return true
     *      to let the iterator return the object to the client.
     */
    protected abstract boolean allows( Object o );
    
    public boolean hasNext() {
        while(next==null && core.hasNext()) {
            // fetch next
            Object o = core.next();
            if( allows(o) )
                next = o;
        }
        return next!=null;
    }

    public Object next() {
        Object r = next;
        next = null;
        return r;
    }

    public void remove() {
        throw new UnsupportedOperationException();
    }
}
